package atomfeed

import (
	"encoding/xml"
	"errors"
	"fmt"
)

type Entry struct {
	*CommonAttributes
	Authors      []*Person           `xml:"author,omitempty"`
	Categories   []*Category         `xml:"category,omitempty"`
	Content      *Content            `xml:"content,omitempty"`
	Contributors []*Person           `xml:"contributors,omitempty"`
	ID           *ID                 `xml:"id"`
	Links        []*Link             `xml:"link,omitempty"`
	Published    *Date               `xml:"published,omitempty"`
	Rights       *Text               `xml:"rights,omitempty"`
	Source       *Source             `xml:"source,omitempty"`
	Summary      *Text               `xml:"summary,omitempty"`
	Title        *Text               `xml:"title"`
	Updated      *Date               `xml:"updated"`
	Extensions   []*ExtensionElement `xml:",any,omitempty"`
}

func (e *Entry) checkAuthors() error {
	if e.Authors == nil {
		if e.Source.Authors == nil {
			return errors.New("no authors set in entry")
		}
	} else {
		for i, a := range e.Authors {
			if err := a.Check(); err != nil {
				return fmt.Errorf("author element %v of entry: %v", i, err)
			}
		}
	}

	return nil
}

func (e *Entry) Check() error {
	if e.ID == nil {
		return errors.New("no id element of entry")
	} else {
		if err := e.ID.Check(); err != nil {
			return fmt.Errorf("id element of entry: %v", err)
		}
	}

	if err := e.checkAuthors(); err != nil {
		return fmt.Errorf("entry %v: %v", e.ID.URI, err)
	}

	if e.Categories != nil {
		for i, c := range e.Categories {
			if err := c.Check(); err != nil {
				return fmt.Errorf("category element %v of entry %v: %v", i, e.ID.URI, err)
			}
		}
	}

	if e.Content != nil {
		if err := (*e.Content).Check(); err != nil {
			return fmt.Errorf("content element of entry %v: %v", e.ID.URI, err)
		}
	}

	if e.Contributors != nil {
		for i, c := range e.Contributors {
			if err := c.Check(); err != nil {
				return fmt.Errorf("contributor element %v of entry %v: %v", i, e.ID.URI, err)
			}
		}
	}

	if e.Links != nil {
		for i, l := range e.Links {
			if err := l.Check(); err != nil {
				return fmt.Errorf("link element %v of entry %v: %v", i, e.ID.URI, err)
			}
		}
	}

	if e.Published != nil {
		if err := e.Published.Check(); err != nil {
			return fmt.Errorf("published element of entry %v: %v", e.ID.URI, err)
		}
	}

	if e.Rights != nil {
		if err := (*e.Rights).Check(); err != nil {
			return fmt.Errorf("rights element of entry %v: %v", e.ID.URI, err)
		}
	}

	if e.Source != nil {
		if err := e.Source.Check(); err != nil {
			return fmt.Errorf("source element of entry %v: %v", e.ID.URI, err)
		}
	}

	if e.Summary != nil {
		if err := (*e.Summary).Check(); err != nil {
			return fmt.Errorf("summary element of entry %v: %v", e.ID.URI, err)
		}
	}

	if e.Title == nil {
		return fmt.Errorf("no title element of entry %v", e.ID.URI)
	} else {
		if err := (*e.Title).Check(); err != nil {
			return fmt.Errorf("title element of entry %v: %v", e.ID.URI, err)
		}
	}

	if e.Updated == nil {
		return fmt.Errorf("no updated element of entry %v", e.ID.URI)
	} else {
		if err := e.Updated.Check(); err != nil {
			return fmt.Errorf("updated element of entry %v: %v", e.ID.URI, err)
		}
	}

	if e.Extensions != nil {
		for i, x := range e.Extensions {
			if err := x.Check(); err != nil {
				return fmt.Errorf("extension element %v of entry %v: %v", i, e.ID.URI, err)
			}
		}
	}

	return nil
}

func (e *Entry) AddExtension(name string, value any) {
	e.Extensions = append(e.Extensions, &ExtensionElement{XMLName: xml.Name{Local: name}, Value: value})
}
